/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Hultmann Extensions.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ['PUBLIC_replaceCore'];
Components.utils.import("resource://hultmann-js-migre-me/util.js");

function PUBLIC_replaceCore(cmdData, newUri, browser) {
  var newUrl = newUri.spec;
  var rv;
  if (cmdData.selectionStart !== cmdData.selectionEnd) {
    //selection
    var txt = cmdData.node.value;
    rv = {
      count: 1,
      text: txt.substr(0, cmdData.selectionStart) + newUrl + txt.substr(cmdData.selectionEnd),
      caretStart: cmdData.selectionStart,
      caretEnd: cmdData.selectionStart + newUrl.length
    };
  } else {
    rv = replaceAll(cmdData, newUrl);
  }


  if (updateNode(cmdData, rv)) {
    // matched text/node/tab not found
    Components.utils.import("resource://hultmann-js-migre-me/err.js");
    var box = util3.getNotification(browser);
    util3.showNotFoundNotification(box, newUrl);
  }
}


function replaceAll(cmdData, newUrl) {
  var inp = cmdData.node;
  var txt = inp.value;
  var qty = 0;
  var idxStart = 0;

  do {
    var idxUrl = txt.indexOf(cmdData.typed, idxStart);
    if (idxUrl > -1) {
      var wordIdx = util.findCurrentUrlPos(txt, idxUrl);
      var urlFound = txt.substr(wordIdx[0], wordIdx[1]);
      //util.log('found ['+wordIdx[0]+', '+wordIdx[1]+' '+urlFound+']');
      if (urlFound === cmdData.typed) {
        txt = txt.substr(0, wordIdx[0]) + newUrl + txt.substr(wordIdx[0] + wordIdx[1]);
        idxStart = wordIdx[0] + newUrl.length;
        qty++;
      } else {
        //util.log('NOT replaced\n'+urlFound +' != '+ cmdData.typed);
        idxStart = wordIdx[0] + wordIdx[1];
      }
    }
  } while (idxUrl > -1);


  return {
    count: qty,
    text: txt,
    caretStart: idxStart,
    caretEnd: idxStart
  };
}


function updateNode(cmdData, rv) {
  if (rv.count == 0) {
    return true;
  }

  if ('rows' in cmdData.node) {
    var pos = cmdData.node.scrollTop;
    cmdData.node.value = rv.text;
    cmdData.node.scrollTop = pos; // restore scrollbar position
  } else {
    cmdData.node.value = rv.text;
  }
  try {
    cmdData.node.setSelectionRange(rv.caretStart, rv.caretEnd);
  } catch(ex) {
    // node/page is no longer available
    return true;
  }

  return false;
}

